# Copyright (c) 2019  Peter Pentchev <roam@ringlet.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

""" Some trivial tests for the trivial repr handler. """

import trivrepr


class Base(trivrepr.TrivialRepr):
    """ A test class. """

    def __init__(self, num):
        # type: (Base, int) -> None
        """ Store a number into an object. """
        super(Base, self).__init__()
        self.num = num

    def increment(self):
        # type: (Base) -> None
        """ Increment the stored number. """
        self.num = self.num + 1

    def query(self):
        # type: (Base) -> int
        """ Return the stored number. """
        return self.num


class Derived(Base):
    """ Another test class. """

    def __init__(self, num, stri):
        # type: (Derived, int, str) -> None
        """ Store a number and a string into an object. """
        super(Derived, self).__init__(num)
        self.stri = stri

    def trim(self):
        # type: (Derived) -> None
        """ Remove whitespace from the start and end of the string. """
        self.stri = self.stri.strip()

    def announce(self):
        # type: (Derived) -> str
        """ Return the string and the number. """
        return '{stri}: {num}'.format(stri=self.stri, num=self.num)


def test_base():
    # type: () -> None
    """ Test the handling of the base class. """
    base = Base(17)
    assert base.query() == 17

    rep = repr(base)
    assert rep.startswith('Base(')
    assert 'num=17' in rep
    assert ',' not in rep

    base.increment()
    assert base.query() == 18

    rep = repr(base)
    assert rep.startswith('Base(')
    assert 'num=18' in rep
    assert ',' not in rep


def test_derived():
    # type: () -> None
    """ Test the handling of the derived class. """
    untrimmed = ' hi \t'
    trimmed = untrimmed.strip()

    derived = Derived(17, untrimmed)
    assert derived.query() == 17
    assert derived.announce() == untrimmed + ': 17'

    rep = repr(derived)
    assert rep.startswith('Derived(')
    assert 'num=17' in rep
    assert 'stri={stri}'.format(stri=repr(untrimmed)) in rep
    assert len(rep.split(', ')) == 2

    derived.increment()
    derived.trim()
    assert derived.query() == 18
    assert derived.announce() == trimmed + ': 18'

    rep = repr(derived)
    assert rep.startswith('Derived(')
    assert 'num=18' in rep
    assert 'stri={stri}'.format(stri=repr(trimmed)) in rep
    assert len(rep.split(', ')) == 2



#!/usr/bin/python
#
# Copyright (c) 2019  Peter Pentchev <roam@ringlet.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

""" Get the change log out of the README file. """

from __future__ import print_function


import re
import sys


RE_VERSION = re.compile(r'''
    ^
    \#\# \s+ Version \s history
    \n+
    (?P<contents>
        .*?
    )
    \n+
    (?:
         ^
        \#\# \s+
        .*
            |
        \Z
    )
''', re.M | re.S | re.X)


def main():
    # type: () -> None
    """ Main program: parse the README file, output the change log. """
    with open('README.md', mode='r') as inf:
        contents = inf.read()

    data = RE_VERSION.search(contents)
    if not data:
        sys.exit('Could not parse README.md')

    with open('NEWS.md', mode='w') as outf:
        print('# Change log for trivrepr\n', file=outf)
        print(data.group('contents'), file=outf)


if __name__ == '__main__':
    main()



# Copyright (c) 2019  Peter Pentchev <roam@ringlet.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

""" Helper module for constructing repr() strings.

The trivrepr module contains the TrivialRepr class that may be
derived from to provide an implementation of __repr__() for
simple classes that have all their attributes passed directly to
the constructor.

    import trivrepr

    class KeyValue(trivrepr.TrivialRepr):
        def __init__(self, key, value, more='hi'):
            super(KeyValue, self).__init__()
            self.key = key
            self.value = value
            self.more = more

    kv = KeyValue(key='key', value='val')
    print(repr(kv))

This program will output:

    KeyValue(key='key', value='val', more='hi')

The trivrepr module is fully typed. """


import inspect


try:
    from typing import List

    _TYPING_USED = (List,)
except ImportError:
    pass


VERSION = '0.1.1'

_GET_SPEC = getattr(inspect, 'getfullargspec', getattr(inspect, 'getargspec'))


class TrivialRepr(object):
    # pylint: disable=too-few-public-methods
    """ Helper class that generates a repr() string.

    Derived classes should take care that all arguments to the __init__()
    method correspond to object attributes with exactly the same names. """

    def __repr__(self):
        # type: (TrivialRepr) -> str
        """ Provide a Python-esque representation of the object. """
        attrlist = _GET_SPEC(self.__init__).args[1:]  # type: ignore
        return '{tname}({attrs})'.format(
            tname=type(self).__name__,
            attrs=', '.join([
                '{name}={value}'.format(name=name,
                                        value=repr(getattr(self, name)))
                for name in attrlist
            ]))



# Copyright (c) 2019  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

""" Packaging metadata for the trivrepr module. """

import re
import setuptools


RE_VERSION = r'''^
    \s* VERSION \s* = \s* '
    (?P<version>
           (?: 0 | [1-9][0-9]* )    # major
        \. (?: 0 | [1-9][0-9]* )    # minor
        \. (?: 0 | [1-9][0-9]* )    # patchlevel
    (?: \. [a-zA-Z0-9]+ )?          # optional addendum (dev1, beta3, etc.)
    )
    ' \s*
    $'''


def get_version():
    """ Get the module version from its __init__.py file. """
    found = None
    re_semver = re.compile(RE_VERSION, re.X)
    with open('trivrepr/__init__.py') as verfile:
        for line in verfile.readlines():
            match = re_semver.match(line)
            if not match:
                continue
            assert found is None
            found = match.group('version')

    assert found is not None
    return found


def get_long_description():
    """ Read the long description from the README.md file. """
    with open('README.md') as descfile:
        return descfile.read()


setuptools.setup(
    name='trivrepr',
    version=get_version(),

    description='Provide a helper for creating repr() strings',
    long_description=get_long_description(),
    long_description_content_type='text/markdown',

    setup_requires=['setuptools-git >= 0.3'],

    author='Peter Pentchev',
    author_email='roam@ringlet.net',
    url='https://devel.ringlet.net/devel/trivrepr/',

    packages=('trivrepr',),
    package_data={
        'trivrepr': [
            # The PEP 484 typed Python module marker
            'py.typed',
        ],
    },

    license='BSD-2',
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',

        'License :: DFSG approved',
        'License :: Freely Distributable',
        'License :: OSI Approved :: BSD License',

        'Operating System :: OS Independent',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',

        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    zip_safe=True,
)
